#include "Gameplay.h"


using namespace std;

//Hotkey status variables
bool enableFrameTime; //F2
bool wireframeMode; //F3
bool texSamplingLinear; //F4 false == nearest neighbour, true == bilinear
int mipMapQuality = -1; //F5 mipMap -1 == off, 0 == nearest neighbour, 1 == bilinear
bool enableViewFrustumCulling; //F8
bool enableTransparency; //F9

using namespace std;

Gameplay::Gameplay(GLFWwindow* window,int width, int height) :window(window)
{
	this->width = width;
	this->height = height;

	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
	// Enable depth test
	glEnable(GL_DEPTH_TEST);
	// Accept fragment if it closer to the camera than the former one
	glDepthFunc(GL_LESS);
	


	//shader
	shader = new Shader("Nyctophilia/Shader/basic.vert", "Nyctophilia/Shader/basic.frag");
	shaderList.push_back(shader);
	Shader* boneshader = new Shader("Nyctophilia/Shader/skinning.vert", "Nyctophilia/Shader/basic.frag");
	shaderList.push_back(boneshader);
	Shader* skyshader = new Shader("Nyctophilia/Shader/basic.vert", "Nyctophilia/Shader/sky.frag");
	shaderList.push_back(skyshader);
	depthShader = new Shader("Nyctophilia/Shader/DepthRTT.vertexshader", "Nyctophilia/Shader/DepthRTT.fragmentshader");
	shaderList.push_back(depthShader);

	//FrustumCuller
	frustumCuller = new FrustumG();
	
	//Postprocessing
	fbo = new FrameBufferObject(width, height);
	flareshader = new Shader("Nyctophilia/Shader/lensFlare.vert", "Nyctophilia/Shader/lensFlare.frag");
	shaderList.push_back(flareshader);
	screenPlane = new ScreenPlane(flareshader, fbo->getColorTexture(), fbo->getDepthTexture());

	//gameplay
	magic = 20000.0; //Time to fly in seconds
	maxHealth = 2.0; //Time allowed to be seen by kids
	health = maxHealth; //Time seen by kids
	
	//Objects
	PhysicWorld::Instance();
	camera = new Camera(glm::mat4(1.0f), window);
	ground = new SkyBox((std::string)"Model/Object/ground20.obj", glm::mat4(1.0f), *shader, (std::string) "Model/Image/ground19.jpg");
		//Ambient
		sky = new SkyBox((std::string)"Model/Object/skybox.obj", glm::mat4(1.0f), *skyshader, (std::string) "Model/Image/skybox2.jpg");
		dlight = new DirectionalLight(glm::vec3(1, 1, 1), glm::translate(glm::mat4(1.0f), glm::vec3(100, 100, 100)), shaderList);
		dlight->draw();

		//Kids
		AnimatedObject* refKid = new AnimatedObject((std::string)"Model/Object/animTest2.dae", glm::mat4(), *boneshader, (std::string)"Model/Object/boy_smooth.png", *camera);
		for (int i = 0; i < 10; i++){
			kids.push_back(new AnimatedObject(*refKid));
			PointLight* plight0 = new PointLight(glm::mat4(), *shader, i);
			pLights.push_back(plight0);
		}
	
		//Trees
		std::cout << "Loading Model: Tree 1" << std::endl;
		Object* refTree1 = new Tree((std::string)"Model/Object/tree1.obj", glm::mat4(1.0f), *shader, (std::string) "Model/Image/tree1.png");
		for (int i = 0; i < 40; i++){
			trees.push_back(new Tree(*refTree1));
		}
		/*std::cout << "Loading Model: Tree 2" << std::endl;
		Object* refTree2 = new Tree((std::string)"Model/Object/tree2.obj", glm::mat4(1.0f), *shader, (std::string) "Model/Image/tree1.png");
		for (int i = 0; i < 30; i++){
			trees.push_back(new Tree(*refTree2));
		}*/
		std::cout << "Loading Model: Tree 3"<< std::endl;
		Object* refTree3= new Tree((std::string)"Model/Object/tree3.obj", glm::mat4(1.0f), *shader, (std::string) "Model/Image/tree1.png");
		for (int i = 0; i < 30; i++){
			trees.push_back(new Tree(*refTree3));
		}

	//Shadows
		// The framebuffer, which regroups 0, 1, or more textures, and 0 or 1 depth buffer.
		FramebufferName = 0;
		glGenFramebuffers(1, &FramebufferName);
		glBindFramebuffer(GL_FRAMEBUFFER, FramebufferName);

		// Depth texture. Slower than a depth buffer, but you can sample it later in your shader
		depthTexture;
		glGenTextures(1, &depthTexture);
		glBindTexture(GL_TEXTURE_2D, depthTexture);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT16, 4096, 4096, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC, GL_LEQUAL);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE, GL_COMPARE_REF_TO_TEXTURE);

		glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, depthTexture, 0);

		// No color output in the bound framebuffer, only depth.	
		glDrawBuffer(GL_NONE);

		// Always check that our framebuffer is ok
		if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
			std::cout <<"error: Frame Buffer not completed" << std::endl;

		
		
}

Gameplay::~Gameplay()
{
	this->cleanup();
}


void Gameplay::draw()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glGenerateMipmap(GL_TEXTURE_2D);
	


	//FIRST RENDER PASS - DEPTH
		
		// Render to our framebuffer
		glBindFramebuffer(GL_FRAMEBUFFER, FramebufferName);
		glViewport(0, 0, 4096, 4096); // Render on the whole framebuffer, complete from the lower left corner to the upper right

		// We don't use bias in the shader, but instead we draw back faces, 
		// which are already separated from the front faces by a small distance 
		// (if your geometry is made this way)
		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK); // Cull back-facing triangles -> draw only front-facing triangles

		// Clear the screen
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		// Use our shader
		depthShader->useShader();

		glEnable(GL_POLYGON_OFFSET_FILL);
		glPolygonOffset(4.0f, 2.0f);


		// Compute the MVP matrix from the light's point of view
		glm::mat4 depthProjectionMatrix = glm::ortho<float>(-100, 100, -100, 100, -100, 200);
		glm::mat4 depthViewMatrix = glm::lookAt(glm::vec3(0, 0, 0), -dlight->getLightDirection(), glm::vec3(0,1, 0));
		glm::mat4 depth_view_proj = depthProjectionMatrix * depthViewMatrix;
		
		
		glUniformMatrix4fv(glGetUniformLocation(depthShader->programHandle, "depth_view_proj"), 1, GL_FALSE, glm::value_ptr(depth_view_proj));
		//for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
		//	(*kid)->draw();
		//}

		ground->setShader(depthShader);
		ground->draw();

		for (std::vector<Tree*>::iterator tree = trees.begin(); tree != trees.end(); ++tree){
			(*tree)->setShader(depthShader);
			(*tree)->draw();
		}

		
		
	// SECOND RENDER PASS - COLORS

		// Use our shader
		shader->useShader();

		glm::mat4 biasMatrix(
			0.5, 0.0, 0.0, 0.0,
			0.0, 0.5, 0.0, 0.0,
			0.0, 0.0, 0.5, 0.0,
			0.5, 0.5, 0.5, 1.0
			);
		glm::mat4 biasVP = biasMatrix * depth_view_proj;

		//glUniformMatrix4fv(glGetUniformLocation(depthShader->programHandle, "biasVP"), 1, GL_FALSE, glm::value_ptr(biasVP));
		
		// Set our "myTextureSampler" sampler to user Texture Unit 0
		glUniform1i(glGetUniformLocation(shader->programHandle, "myTextureSampler"), 0);

		glActiveTexture(GL_TEXTURE1);
		glBindTexture(GL_TEXTURE_2D, depthTexture);
		glUniform1i(glGetUniformLocation(shader->programHandle, "shadowMap") , 1);

		//Bind Shader Results to Frame Buffer
		fbo->bind();
		//glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glViewport(0, 0, width, height); // Render on the whole framebuffer, complete from the lower left corner to the upper right
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // clear buffers

		glUniformMatrix4fv(glGetUniformLocation(shader->programHandle, "biasVP"), 1, GL_FALSE, glm::value_ptr(biasVP));
		for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
			(*kid)->draw();
		}

		dlight->draw();

		drawnObjects = trees.size();
		for (std::vector<Tree*>::iterator tree = trees.begin(); tree != trees.end(); ++tree){
			/*std::cout << (*tree)->centerOfVertices.x << " "
			<< (*tree)->centerOfVertices.y << " "
			<< (*tree)->centerOfVertices.z << " "
			<< (*tree)->boundingSphereRadius << " "
			<< frustumCuller->sphereInFrustum((*tree)->centerOfVertices, (*tree)->boundingSphereRadius) << std::endl; */
			if (frustumCuller->sphereInFrustum((*tree)->centerOfVertices, (*tree)->boundingSphereRadius) != frustumCuller->OUTSIDE){
			}
			else{
				drawnObjects--;
			}
			(*tree)->setShader(shader);
			(*tree)->draw();
		}

		sky->draw();

		ground->setShader(shader);
		shader->useShader();
		glUniformMatrix4fv(glGetUniformLocation(shader->programHandle, "biasVP"), 1, GL_FALSE, glm::value_ptr(biasVP));
		ground->draw();

	// THIRD RENDER PASS - SCREEN PLANE, FLARES & HEALTH
		
		glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		//Bind Shader Result to ViewPort
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		// clear buffers
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		float healthPercentage = health / maxHealth;
	
		screenPlane->draw(healthPercentage);
		

		if (texSamplingLinear && mipMapQuality == -1) //Bilinear / MipMap OFF
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		}
		else if (!texSamplingLinear && mipMapQuality == -1) //Nearest Neighbour / MipMap OFF
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		}
		else if (texSamplingLinear && mipMapQuality == 0) //Bilinear / MipMap Nearest Neighbour
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		}
		else if (!texSamplingLinear && mipMapQuality == 0) //Nearest Neighbour / MipMap Nearest Neighbour
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
		}
		else if (texSamplingLinear && mipMapQuality == 1) //Bilinear / MipMap Bilinear
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		}
		else if (!texSamplingLinear && mipMapQuality == 1) //Nearest Neighbour / MipMap Bilinear
		{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		}
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

		if (wireframeMode){
			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
		}
}


void Gameplay::update(GLFWwindow* window, float time_delta)
{
	health += 0.1*time_delta*maxHealth;
	if (health > maxHealth){ health = maxHealth; }
	if (health < 0) { health = 0; }
	magic += 0.05*time_delta;
	//if (magic > 2){ magic = 2.0f; }

	camera->update(time_delta);
	if (glfwGetKey(window, GLFW_KEY_SPACE) == GLFW_PRESS && magic >= time_delta){
		btVector3 force = btVector3(0, 15, 0);
		camera->body->applyImpulse(force, btVector3(0.3, 0.1, 0.3));
		magic -= time_delta;
	}

	ground->update(time_delta);

	this->perspective();

	for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
		(*kid)->update(time_delta);
	}
	
	for (std::vector<Tree*>::iterator tree = trees.begin(); tree != trees.end(); ++tree){
		(*tree)->update(time_delta);
	}
	
	updateKidsLight();

	//Win & Loose Condition
	for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
		int interactionResult = (*kid)->interactionWithCamera(window);
		if (interactionResult == -1 && (*kid)->isAlive()){
			//kid has seen player -> lose health
			health = health - time_delta;
			if (health <= 0){
				std::cout << "GAME OVER! The kid has seen you. (Press enter to close)" << std::endl;
				std::cin.get();
				glfwSetWindowShouldClose(window, true);
			}
		}
		else if (interactionResult == 1){
			//player screamed in range -> terrify kid
			if ((*kid)->isAlive()){
				(*kid)->setAlive(false);
				
				int kidsAlive = kids.size();
				for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
					if ((*kid)->isAlive() == false){
						kidsAlive--;
					}
				}
				if (kidsAlive == 0){
					cout << "All kids are terrified! You won! (Press enter to close)" << endl;
					std::cin.get();
					glfwSetWindowShouldClose(window, true);
				}
				else{
					cout << "Terrified kid! Kids left: " << kidsAlive << " of " << kids.size() << endl;
				}
			}
		}
	}

	PhysicWorld::update(time_delta);
}

void Gameplay::perspective(){

	auto proj = glm::perspective(glm::radians(60.0f), width / (float)height, 0.1f, 1000.0f);
	auto view = camera->getViewMatrix();
	auto view_proj = proj * view;
	for (std::vector<Shader*>::iterator shaderIter = shaderList.begin(); shaderIter != shaderList.end(); ++shaderIter){
		(*shaderIter)->useShader();
		glUniformMatrix4fv(glGetUniformLocation((*shaderIter)->programHandle, "view_proj"), 1, GL_FALSE, glm::value_ptr(view_proj));
	}
}

void Gameplay::updateKidsLight(){
	

	float * light_positions = (float*)malloc(sizeof(float) * 3 * pLights.size());
	
	for (std::vector<Shader*>::iterator shaderIter = shaderList.begin(); shaderIter != shaderList.end(); ++shaderIter){
		unsigned int lightCount = 0;
		(*shaderIter)->useShader();
		
		for (std::vector<AnimatedObject*>::iterator kid = kids.begin(); kid != kids.end(); ++kid){
			if ((*kid)->isAlive() == true){
				light_positions[0 + lightCount*3] = (*kid)->modelMatrix[3].x + 0.6;
				light_positions[1 + lightCount*3] = (*kid)->modelMatrix[3].y + 0.8;
				light_positions[2 + lightCount*3] = (*kid)->modelMatrix[3].z;
				lightCount++;
			}
		}
		glUniform3fv(glGetUniformLocation((*shaderIter)->programHandle, "pLightPos"), lightCount, light_positions);
		glUniform1i(glGetUniformLocation((*shaderIter)->programHandle, "pLightCount"), lightCount);
	}
}

void Gameplay::readControlKeys(){
	if (glfwGetKey(window, GLFW_KEY_ESCAPE)){
		glfwSetWindowShouldClose(window, true);
	}
	if (glfwGetKey(window, GLFW_KEY_F2))
	{
		cout << "Frighten all Kids without being seen by them!" << endl;
		cout << "WASD - Move" << endl;
		cout << "Ctrl - Scream (to frighten a Kid if you are behind it)" << endl;
		cout << "Space - Fly (as long as you have enough magic)" << endl;
		cout << "Esc - End game" << endl;
		cout << "F1 - Help" << endl;
		cout << "F2 - Show Frametime" << endl;
		cout << "F3 - Wireframemode (On/Off)" << endl;
		cout << "F4 - Texture Samping (Bilinear/Nearest Neighbour)" << endl;
		cout << "F5 - Mipmap Mode (Off/Nearest Neighbour/Bilinear)" << endl;
		cout << "F8 - Viewfrustum - Culling (On/Off)" << endl;
		cout << "F9 - Transparency (On/Off)" << endl;
	}
	if (glfwGetKey(window, GLFW_KEY_F2))
	{
		enableFrameTime = !enableFrameTime;
	}
	if (glfwGetKey(window, GLFW_KEY_F3))
	{
		wireframeMode = !wireframeMode;
		if (wireframeMode)
		{
			std::cout << "enable wireframe mode" << std::endl;
		}
		else {
			std::cout << "disable wireframe mode" << std::endl;
		}
	}
	if (glfwGetKey(window, GLFW_KEY_F4))
	{
		texSamplingLinear = !texSamplingLinear;
		if (texSamplingLinear)
		{
			std::cout << "texture sampling bilinear" << std::endl;
		}
		else {
			std::cout << "texture sampling nearest neighbour" << std::endl;
		}
	}
	if (glfwGetKey(window, GLFW_KEY_F5))
	{
		mipMapQuality++;
		if (mipMapQuality > 1)
		{
			mipMapQuality = -1;
		}

		if (mipMapQuality == -1)
		{
			std::cout << "mipmap off" << std::endl;
		}
		else if (mipMapQuality == 0)
		{
			std::cout << "mipmap nearest neighbour" << std::endl;
		}
		else if (mipMapQuality == 1)
		{
			std::cout << "mipmap bilinear" << std::endl;
		}
	}
	if (glfwGetKey(window, GLFW_KEY_F8))
	{
		enableViewFrustumCulling = !enableViewFrustumCulling;
		if (enableViewFrustumCulling)
		{
			std::cout << "enable View Frustum Culling" << std::endl;
		}
		else {
			std::cout << "disable View Frustum Culling" << std::endl;
		}
	}
	if (glfwGetKey(window, GLFW_KEY_F9))
	{
		enableTransparency = !enableTransparency;
		if (enableTransparency)
		{
			std::cout << "enable transparency" << std::endl;
			screenPlane->setTransparency(true);
		}
		else {
			std::cout << "disable transparency" << std::endl;
			screenPlane->setTransparency(false);
		}
	}
}

void Gameplay::runProgram()
{
	double time = glfwGetTime();
	double lastPrintTick = glfwGetTime();

	while (!glfwWindowShouldClose(window))
	{
		double time_new = glfwGetTime();
		float time_delta = (float)(time_new - time);
		time = time_new;

		if ((glfwGetTime() - lastPrintTick) > 0.1){
			readControlKeys();
			if (enableFrameTime){
				cout << (int)(1 / time_delta) << " FPS / Frametime: " << time_delta << "s" << endl;
			}
			if (enableViewFrustumCulling){
				std::cout << "Objects drawn: " << drawnObjects + 12 << std::endl;
			}
			lastPrintTick = glfwGetTime();
		}



		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		update(window, time_delta);
		draw();

		glfwSwapBuffers(window);
		glfwPollEvents();


		while (glGetError() != GL_NO_ERROR) {
			std::cout << "ERROR: OpenGL!" << std::endl;
#if _DEBUG
			//system("PAUSE");
#endif
		}
	}
	this->cleanup();
}

void Gameplay::cleanup()
{
	delete dlight; dlight = nullptr;
	delete sky; sky = nullptr;
	delete ground; ground = nullptr;
	trees.clear();
	pLights.clear();
	kids.clear();
	shaderList.clear();
}

void Gameplay::drawMagicMar(){
	glBegin(GL_QUADS);
	glColor3f(1, 0, 0);
	glVertex2f(0, 0);
	glVertex2f(1, 0);
	glColor3f(magic, 0, 0);
	glVertex2f(1, magic);
	glVertex2f(0, magic);
	glVertex2f(0, magic);
	glVertex2f(1, magic);
	glColor3f(0, 0, 0);
	glVertex2f(1, 1);
	glVertex2f(0, 1);
	glEnd();
}