#pragma once

#include "Scene/Texture.h"
#include "Shader/Shader.h"
#include "Scene/SceneObject.h"
#include "Scene/Camera.h"
#include "Scene/Object.h"
#include "Physics/PhysicWorld.h"
#include "Scene/DirectionalLight.h"
#include "btBulletDynamicsCommon.h"
#include "Physics/PhysicWorld.h"
#include "Scene/Tree.h"
#include "Scene/PointLight.h"
#include <list>
#include "Scene/SkyBox.h"
#include "Scene/AnimatedObject.h"
#include "Shader/ScreenPlane.h"
#include "FrustumG.h"
#include "Shader/FrameBufferObject.h"
#include <glm/gtc/matrix_transform.hpp>

using namespace nycto;
using namespace nycto::scene;
using namespace nycto::shader;

class Gameplay
{
public:
	Gameplay(GLFWwindow* window, int width, int height);
	~Gameplay();
	void runProgram();

private:
	void cleanup();
	void draw();
	void update(GLFWwindow* window, float time_delta);
	void readControlKeys();
	void updateKidsLight();
	void drawMagicMar();
	GLFWwindow* window;
	FrameBufferObject* fbo;

	Camera* camera;
	DirectionalLight* dlight;

	Shader* depthShader;
	Shader* shader;
	Shader* skyshader;
	Shader* flareshader;
	Shader* boneshader;
	Shader* depthBoneshader;
	GLuint FramebufferName;
	GLuint depthTexture;
	FrustumG* frustumCuller;
	int drawnObjects = 0;

	Object* sky;
	Object* ground;
	ScreenPlane* screenPlane;
	std::vector<Tree*> trees;
	std::vector<PointLight*> pLights;
	std::vector<AnimatedObject*> kids;
	std::vector<Shader*> shaderList;
	int width = 1200;
	int height = 900;

	float magic;
	float health;
	float maxHealth;

	void perspective();
};