#include "PhysicWorld.h"
#include <iostream>
using namespace nycto;
using namespace nycto::scene;

btDiscreteDynamicsWorld* PhysicWorld::dynamicsWorld = NULL;
Shader* PhysicWorld::shader = NULL;
GLDebugDrawer* PhysicWorld::debugDrawer = NULL;

btDiscreteDynamicsWorld* PhysicWorld::Instance()
{
	if (!dynamicsWorld)
	{
		btBroadphaseInterface* broadphase = new btDbvtBroadphase();
		btDefaultCollisionConfiguration* collisionConfiguration = new btDefaultCollisionConfiguration();
		btCollisionDispatcher* dispatcher = new btCollisionDispatcher(collisionConfiguration);
		btSequentialImpulseConstraintSolver* solver = new btSequentialImpulseConstraintSolver;
		dynamicsWorld = new btDiscreteDynamicsWorld(dispatcher, broadphase, solver, collisionConfiguration);
		dynamicsWorld->setGravity(btVector3(0, -200, 0));
		//collision

#if _DEBUG
		shader = new Shader("Nyctophilia/Shader/debugShader.vert", "Nyctophilia/Shader/debugShader.frag");
		debugDrawer = new GLDebugDrawer(shader);
		debugDrawer->setDebugMode(1);
		dynamicsWorld->setDebugDrawer(debugDrawer);
#endif
	}
	return dynamicsWorld;
}

glm::mat4 PhysicWorld::convertBulletTransformToGLM(btTransform& transform)
{
	float data[16];
	transform.getOpenGLMatrix(data);
	return glm::make_mat4(data);
}

btTransform PhysicWorld::convertGLMTransformToBullet(glm::mat4& transform)
{
	const float* data = glm::value_ptr(transform);
	btTransform bulletTransform;
	bulletTransform.setFromOpenGLMatrix(data);
	return bulletTransform;
}

btVector3 PhysicWorld::convertGLMVectorToBullet(glm::vec3 vector)
{
	return btVector3(vector.x, vector.y, vector.z);
}

glm::vec3 PhysicWorld::convertBulletVectorToGLM(btVector3& vector)
{
	return glm::vec3(vector.getX(), vector.getY(), vector.getZ());
}

void PhysicWorld::update(float deltaT)
{
	dynamicsWorld->stepSimulation(deltaT, 10);
#if _DEBUG

	//dynamicsWorld->stepSimulation(deltaT, 10);
	//dynamicsWorld->debugDrawWorld();
	//debugDrawer->drawAll();

	//shader->useShader();
#endif
}

PhysicWorld::~PhysicWorld()
{
	delete dynamicsWorld;
	dynamicsWorld = nullptr;
	delete debugDrawer;
	debugDrawer = nullptr;
	delete shader;
	shader = nullptr;
}

btRigidBody* PhysicWorld::createCylinder(float radius, float height, float px, float py, float pz, float mass)
{
	//ref:http://stackoverflow.com/questions/9117932/detecting-collisions-with-bullet

	btCollisionShape* colShape = new btCylinderShape(btVector3(radius, height, height));

	return PhysicWorld::addCollisionShape(colShape, px, py, pz, mass);
}

btRigidBody* PhysicWorld::createPlane(float width, float height, float px, float py, float pz, float mass)
{
	//ref:http://stackoverflow.com/questions/9117932/detecting-collisions-with-bullet

	btCollisionShape* colShape = new btStaticPlaneShape(btVector3(0, 1, 0), 1);

	return PhysicWorld::addCollisionShape(colShape, px, py, pz, mass);
}

btRigidBody* PhysicWorld::addCollisionShape(btCollisionShape* colShape, float px, float py, float pz, float mass)
{
	btTransform startTransform;
	startTransform.setIdentity();

	btScalar    tMass(mass);

	//rigidbody is dynamic if and only if mass is non zero, otherwise static
	bool isDynamic = (tMass != 0.f);

	btVector3 localInertia(0, 0, 0);
	if (isDynamic)
		colShape->calculateLocalInertia(tMass, localInertia);

	startTransform.setOrigin(btVector3(px, py, pz));

	//using motionstate is recommended, it provides interpolation capabilities, and only synchronizes 'active' objects
	btDefaultMotionState* myMotionState = new btDefaultMotionState(startTransform);
	btRigidBody::btRigidBodyConstructionInfo rbInfo(tMass, myMotionState, colShape, localInertia);
	btRigidBody* body = new btRigidBody(rbInfo);
	dynamicsWorld->addRigidBody(body);
	//collision
	return body;
}