#pragma once

#include "Camera.h"
#include "SceneObject.h"
#include "../Shader/Shader.h"
#include "SkinnedMesh.h"
#include <glm/gtc/type_ptr.hpp>
#include <assimp/Importer.hpp>
#include <assimp/mesh.h>
#include <assimp/postprocess.h>
#include "Texture.h"
#include <glm/gtc/matrix_transform.hpp>
#include <glm/glm.hpp>

#ifndef WIN32
#include <sys/time.h>
#include <unistd.h>
#endif

class SkinnedMesh;

namespace nycto
{
	namespace scene
	{
			class AnimatedObject : public SceneObject
			{
			public:
				AnimatedObject::AnimatedObject(std::string& objectPath, glm::mat4& _matrix, Shader& shader, std::string& texturePath, Camera &camera);
				AnimatedObject::AnimatedObject(AnimatedObject& object);
				virtual ~AnimatedObject();
				virtual void update(float deltaT);
				virtual void draw();
				int interactionWithCamera(GLFWwindow* window); //-1 = lose condition, 0 = nothing, 1 = terrified kid
				virtual void setAlive(bool alive);
				virtual bool isAlive();
			private:

				SkinnedMesh* mesh;
				Shader* shader;
				std::string& objectPath;
				std::string& texturePath;
				float startTime = 0;
				bool alive = true;

				//previous kid.cpp
				void moveKid(float deltaT);
				void moveBones();
				double getKidViewAngle();
				double getKidSeesCameraInAngle();
				
				Camera *camera;
				bool seesCamera();
				double const HEIGHT_OF_KID = 1.2;
				double const RADIUS_OF_KID = 0.9;
				int rotationCounter = 0;
				int sign = 1;
				double rotationLimit = 100;
				void placeTreeRamdomly();
			};
		
	}
}