#include <iostream>
#include "Camera.h"
#include <GLFW/glfw3.h>
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glut/glut.h>
#include "../Physics/PhysicWorld.h"

using namespace nycto;
using namespace nycto::scene;


Camera::Camera(glm::mat4& matrix, GLFWwindow* window)
	: SceneObject(matrix)
{
	this->window = window;
	glfwSetCursorPos(window, glutGet(GLUT_SCREEN_WIDTH) / 2, glutGet(GLUT_SCREEN_HEIGHT) / 2);
	body = PhysicWorld::createCylinder(RADIUS_OF_CAMERA, HEIGHT_OF_CAMERA, -7 /*xpos*/, HEIGHT_OF_CAMERA /*xpos*/, 1 /*zpos*/, 1);
	body->setAngularFactor(btVector3(0, 0, 0));
}


Camera::~Camera()
{
	delete window;
	window = nullptr;
	delete body;
	body = nullptr;
}

void Camera::update(float time_delta){
	// Get mouse position
	double xpos, ypos;
	bool ducked = false;
	glfwGetCursorPos(window, &xpos, &ypos);

	// Reset mouse position for next frame
	glfwSetCursorPos(window, glutGet(GLUT_SCREEN_WIDTH) / 2, glutGet(GLUT_SCREEN_HEIGHT) / 2);

	// Compute new orientation
	horizontalAngle += mouseSpeed * float(glutGet(GLUT_SCREEN_WIDTH) / 2 - xpos);
	verticalAngle += mouseSpeed * float(glutGet(GLUT_SCREEN_HEIGHT) / 2 - ypos);

	// Direction : Spherical coordinates to Cartesian coordinates conversion
	glm::vec3 direction(
		cos(verticalAngle) * sin(horizontalAngle),
		sin(verticalAngle),
		cos(verticalAngle) * cos(horizontalAngle)
		);

	// Right vector
	glm::vec3 right = glm::vec3(
		sin(horizontalAngle - 3.14f / 2.0f),
		0,
		cos(horizontalAngle - 3.14f / 2.0f)
		);



	// Up vector
	glm::vec3 up = glm::cross(right, direction);

	deltaPosition = glm::vec3(0.0f, 0.0f, 0.0f);

	// Move forward
	if (glfwGetKey(window, GLFW_KEY_UP) == GLFW_PRESS || glfwGetKey(window, 'W') == GLFW_PRESS){
		deltaPosition += direction;
	}
	// Move backward
	if (glfwGetKey(window, GLFW_KEY_DOWN) == GLFW_PRESS || glfwGetKey(window, 'S') == GLFW_PRESS){
		deltaPosition -= direction;
	}
	// Strafe right
	if (glfwGetKey(window, GLFW_KEY_RIGHT) == GLFW_PRESS || glfwGetKey(window, 'D') == GLFW_PRESS){
		deltaPosition += right;
	}
	// Strafe left
	if (glfwGetKey(window, GLFW_KEY_LEFT) == GLFW_PRESS || glfwGetKey(window, 'A') == GLFW_PRESS){
		deltaPosition -= right;
	}

	//Shift pressed?
	if (glfwGetKey(window, GLFW_KEY_LEFT_SHIFT) == GLFW_PRESS){
		speed = 8.0f;
	}
	else{
		speed = 3.0f;
	}

	/*
	//Control pressed?
	if (glfwGetKey(window, GLFW_KEY_LEFT_CONTROL) == GLFW_PRESS){
		speed = 1.5f;
		ducked = true;
	}
	*/


	//Make sure player does not speed up if two keys are pressed
	glm::normalize(deltaPosition);
	deltaPosition *= speed;

	body->activate();
	body->setLinearVelocity(btVector3(deltaPosition.x, 0, deltaPosition.z));

	btTransform transform = body->getWorldTransform();
	modelMatrix[3].x = transform.getOrigin().getX();
	modelMatrix[3].y = transform.getOrigin().getY();
	modelMatrix[3].z = transform.getOrigin().getZ();


	float FoV = initialFoV;// - 5 * glfwGetMouseWheel(); // Now GLFW 3 requires setting up a callback for this.

	// Projection matrix : 45 Field of View, 4:3 ratio, display range : 0.1 unit <-> 100 units
	// ProjectionMatrix = glm::perspective(FoV, 4.0f / 3.0f, 0.1f, 100.0f);
	// Camera matrix
	viewMatrix = glm::lookAt(
		glm::vec3(modelMatrix[3].x, modelMatrix[3].y, modelMatrix[3].z),           // Camera is here
		glm::vec3(modelMatrix[3].x, modelMatrix[3].y, modelMatrix[3].z) + direction, // and looks here : at the same position, plus "direction"
		up                  // Head is up (set to 0,-1,0 to look upside-down)
		);
}

glm::mat4 Camera::getViewMatrix(){
	return viewMatrix;
}

glm::vec3 Camera::getPosition(){
	return glm::vec3(modelMatrix[3].x, modelMatrix[3].y, modelMatrix[3].z);

}
