#include "FrameBufferObject.h"

using namespace nycto;

FrameBufferObject::FrameBufferObject(unsigned int width, unsigned int height)
{
	glGenFramebuffers(1, &fbo);
	glBindFramebuffer(GL_FRAMEBUFFER, fbo);

	generateColorTexture(width, height);//generate empty texture
	generateDepthTexture(width, height);//generate empty texture

	//to keep track of our textures
	unsigned int attachment_index_color_texture = 0;

	//bind textures to pipeline. texture_depth is optional
	//0 is the mipmap level. 0 is the heightest

	glFramebufferTexture(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0 + attachment_index_color_texture, texture_color, 0);
	glFramebufferTexture(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, texture_depth, 0); //optional

	//add attachements
	drawbuffer.push_back(GL_COLOR_ATTACHMENT0 + attachment_index_color_texture);
	glDrawBuffers(drawbuffer.size(), &drawbuffer[0]);
	//Check for FBO completeness
	if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE){
		std::cout << "Error! FrameBuffer is not complete" << std::endl;
		std::cin.get();
		std::terminate();
	}

	//unbind framebuffer
	glBindFramebuffer(GL_FRAMEBUFFER, 0);
}

FrameBufferObject::~FrameBufferObject()
{
	glDeleteFramebuffers(1, &fbo);
	glDeleteTextures(1, &texture_color);
	glDeleteTextures(1, &texture_depth);
}

void FrameBufferObject::generateColorTexture(unsigned int width, unsigned int height)
{
	glGenTextures(1, &texture_color);
	glBindTexture(GL_TEXTURE_2D, texture_color);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, 0);
}

void FrameBufferObject::generateDepthTexture(unsigned int width, unsigned int height)
{
	glGenTextures(1, &texture_depth);
	glBindTexture(GL_TEXTURE_2D, texture_depth);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_DEPTH_COMPONENT24, width, height, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
}

GLint FrameBufferObject::getColorTexture()
{
	return texture_color;
}

GLint FrameBufferObject::getDepthTexture()
{
	return texture_depth;
}

void FrameBufferObject::bind()
{
	glBindFramebuffer(GL_FRAMEBUFFER, fbo);
}