#include "ScreenPlane.h"

using namespace nycto;
using namespace nycto::shader;

ScreenPlane::ScreenPlane(Shader* shader, GLuint colorTexture, GLuint depthTexture)
	:shader(shader),
	scene_texture_color(colorTexture),
	scene_texture_depth(depthTexture)
{
	setTransparency(true);
	static const GLfloat positions[] = {
		-1, -1, 0.0f,
		1, -1, 0.0f,
		-1, 1, 0.0f,
		-1, 1, 0.0f,
		1, -1, 0.0f,
		1, 1, 0.0f,
	};

	static const GLfloat uvs[] = {
		0, 0,
		1, 0,
		0, 1,
		0, 1,
		1, 0,
		1, 1
	};

	glGenBuffers(1, &vertexBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, vertexBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(positions), positions, GL_STATIC_DRAW);

	glGenBuffers(1, &uvBuffer);
	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	glBufferData(GL_ARRAY_BUFFER, sizeof(uvs), uvs, GL_STATIC_DRAW);

	glGenVertexArrays(1, &vaoID);
	glBindVertexArray(vaoID);


	glBindVertexArray(vaoID);
	glBindBuffer(GL_ARRAY_BUFFER, vertexBuffer);
	int location_position = glGetAttribLocation(shader->programHandle, "position");
	if (location_position > -1)
	{
		glEnableVertexAttribArray(location_position);
		glVertexAttribPointer(location_position, 3, GL_FLOAT, GL_FALSE, 0, 0);
	}

	glBindBuffer(GL_ARRAY_BUFFER, uvBuffer);
	GLint uvIndex = glGetAttribLocation(shader->programHandle, "uv");
	if (uvIndex > -1)
	{
		glEnableVertexAttribArray(uvIndex);
		glVertexAttribPointer(uvIndex, 2, GL_FLOAT, GL_FALSE, 0, 0);
	}

	gameOverTexture = new Texture(GL_TEXTURE_2D, "Textures/gameOverScreen.png");
	gameWonTexture = new Texture(GL_TEXTURE_2D, "Textures/gameOverScreen.png");
}

ScreenPlane::~ScreenPlane()
{

}

void ScreenPlane::draw(float healthPercentage)
{
	shader->useShader();
	
	glUniform1f(glGetUniformLocation(shader->programHandle, "healthPercentage"), healthPercentage);

	glUniform1i(glGetUniformLocation(shader->programHandle, "sceneTexture"), 0);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, scene_texture_color);

	
	//glUniform1i(glGetUniformLocation(shader->programHandle, "gameOverTexture"), 1);
	//gameOverTexture->Bind(GL_TEXTURE1);
	//glUniform1i(glGetUniformLocation(shader->programHandle, "gameWonTexture"), 2);
	//gameOverTexture->Bind(GL_TEXTURE2);

	/*
	
	GLuint location_lightSourceTexture = glGetUniformLocation(shader->programHandle, "lightSourceTexture");
	glUniform1i(location_lightSourceTexture, 1);
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, sun_texture_color);

	GLuint location_chromaticDistortionTexture = glGetUniformLocation(shader->programHandle, "chromaticDistortionTexture");
	glUniform1i(location_chromaticDistortionTexture, 2);
	chromaticDistortion_texture->Bind(GL_TEXTURE2);
	*/
	glBindVertexArray(vaoID);
	glDrawArrays(GL_TRIANGLES, 0, 6);
	glBindVertexArray(0);
}

void ScreenPlane::setTransparency(bool onOff){
	shader->useShader();
	int temp;
	if (onOff){ temp = 1; }
	else{ temp = 0; }
	glUniform1i(glGetUniformLocation(shader->programHandle, "transparencyFlag"), temp);
}