#include "Shader.h"

#include <iostream>
#include <fstream>

using namespace nycto;
using namespace std;

Shader::Shader(const std::string& vertexShader, const std::string& fragmentShader)
	//standardmaesig auf 0 setzen
	:programHandle(0), vertexHandle(0),fragmentHandle(0)
{
	//programm anlegen
	programHandle = glCreateProgram();

	//check obs funktioniert hat
	if(programHandle == 0)
	{
		std::cout << "ERROR: Could not create Program.." << std::endl;
		exit(EXIT_FAILURE);
	}
	//objekte anlegen und handles zurck bekommen
	loadShader(vertexShader, GL_VERTEX_SHADER,vertexHandle);
	loadShader(fragmentShader, GL_FRAGMENT_SHADER,fragmentHandle);

	//shader linken
	link();
}

Shader::~Shader(){
	glDeleteProgram(programHandle);
	glDeleteShader(vertexHandle);
	glDeleteShader(fragmentHandle);
}

void Shader::useShader() const
{
	glUseProgram(programHandle);
}

void Shader::loadShader(const std::string& shader, GLenum shaderType, GLuint& handle){
	std::ifstream shaderFile(shader);
	if(!shaderFile.good())
	{
		std::cout << "ERROR: Could not load file" << std::endl;
		system("PAUSE");
		exit(EXIT_FAILURE);
	}
	std::string code = std::string(
	std::istreambuf_iterator<char>(shaderFile),
	std::istreambuf_iterator<char>());

	shaderFile.close();
	handle = glCreateShader(shaderType);
	if(handle == 0)
	{
		std::cout << "ERROR: Could not create shader" << std::endl;
		system("PAUSE");
		exit(EXIT_FAILURE);
	}
	const char * codePtr = code.c_str();
	glShaderSource(handle, 1 ,&codePtr,nullptr);
	glCompileShader(handle);

	GLint succeeded;
	glGetShaderiv(handle, GL_COMPILE_STATUS, &succeeded);
	if(!succeeded)
	{
		GLint logSize;
		glGetShaderiv(handle, GL_INFO_LOG_LENGTH, &logSize);

		GLchar* msg = new GLchar[logSize];
		glGetShaderInfoLog(handle, logSize, nullptr, msg);

		std::cout << msg <<" check" << std::endl;
		delete[] msg;
		system("PAUSE");
		exit(EXIT_FAILURE);
	}

}
void Shader::link(){
	glAttachShader(programHandle, vertexHandle);
	glAttachShader(programHandle, fragmentHandle);

	glLinkProgram(programHandle);

	GLint succeeded;
	glGetProgramiv(programHandle, GL_LINK_STATUS, &succeeded);
	if(!succeeded)
	{
		GLint logSize;
		glGetProgramiv(programHandle, GL_INFO_LOG_LENGTH, &logSize);

		GLchar* msg = new GLchar[logSize];
		glGetProgramInfoLog(programHandle, logSize, nullptr, msg);

		std::cout << msg << std::endl;
		delete[] msg;
		system("PAUSE");
		exit(EXIT_FAILURE);
	}


}
